-------------------------------------------------------------------------------------
-- | Welcome, Friend |---------------------------------------------------------------
-------------------------------------------------------------------------------------

-- Thank you for using IgnoreEnemyEmotes!
-- This addon attemps to filter out the messages of the opposing faction to keep the chat clear.
-- This means the Stewards don't have to do it, and they then finally have some time to eat some delicious purians.
-- If you read this, please give the Steward of the Day a nice purian in Bastion!

-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, IEE = ...

-- Shorthand for the IEE.options namespace.
local options = IEE.options

-- Expose the Addon to the Global Space.
-- Everything in this space can be accessed externally through calling IgnoreEnemyEmotes.
IgnoreEnemyEmotes = IEE

-------------------------------------------------------------------------------------
-- | Variable Definitions |----------------------------------------------------------
-------------------------------------------------------------------------------------

-- Races based on their localization independant name.
local hordeRaces = {
	["BloodElf"]	         = 1,
	["Goblin"]		         = 1,
	["HighmountainTauren"] = 1,
	["MagharOrc"]          = 1,
	["Nightborne"]         = 1,
	["Orc"]				         = 1,	
	["Tauren"] 		         = 1,
	["Troll"]			         = 1,
	["Undead"]		         = 1,
	["Scourge"]            = 1,
	["Vulpera"]            = 1,
	["ZandalariTroll"]     = 1,
}

-- Races based on their localization independant name.
local allianceRaces = {
	["DarkIronDwarf"]      = 1,
	["Draenei"]		         = 1,
	["Dwarf"]			         = 1,
	["Gnome"] 		         = 1,
	["Human"]			         = 1,
	["LightforgedDraenei"] = 1,
	["KulTiran"]           = 1,
	["Mechagnome"]         = 1,
	["NightElf"]	         = 1,
	["ThinHuman"]          = 1,
	["VoidElf"]            = 1,
	["Worgen"]		         = 1,
}

-- Set the list of races of the other faction.
local opposingRaces = UnitFactionGroup("player") == "Horde"	and allianceRaces or hordeRaces

-------------------------------------------------------------------------------------
-- | IgnoreEnemyEmotes Filters |-----------------------------------------------------
-------------------------------------------------------------------------------------

-- Returns whether a race is of the opposing faction (or in the case of Pandaren, checks whether they speak an opposing language).
-- There is a difficulty with Pandaren of either faction being treated the same in the game code.
-- As such, there is a fallback filter to check the language of a Pandaren, if it is the shared language of the opposing faction (Common/Orcish) then it can be filtered.
-- There is no simple way of filtering Pandaren speaking their native language, but this should be so infrequent that no workaround is provided in this addon.
local function isOpposingFaction(race, language)
  -- If the race of the sender is from the other faction
  
	if race and opposingRaces[race] then
		return true
	end
  -- Pandaren are complicated as the huojin and tushui pandaren share the pandaren language.
  -- Not too much can be done here, other than to filter out their char if they are using the generic faction language.
	if race == "Pandaren" then
		if UnitFactionGroup("player") == "Horde"    and language == "Common" then
			return true
		end
		if UnitFactionGroup("player") == "Alliance" and language == "Orcish" then
			return true
		end
  end
  return false -- Fallback, return "false", aka no filtering.
end

-- Returns whether we're should filter because we're either in PvP or outside of PvP and filtering should only be in PvP.
local function filterForOutdoorSettings(inPvP)
  if    not (inPvP) and IEE.options.savedVars["onlyInPvP"] then
    return false
  end  -- We are not in pvp, and we shouldn't filter outside pvp, so filter returns 0
  
  -- All other cases, filter.
  return true
end

-- Helper function to detect whether the player is in a BG or Arena.
local function isInBgOrArena()
    -- Fetch the instance type.
  local _, instanceType = IsInInstance()
  return instanceType == "pvp" or instanceType == "arena"
  
end

-- Combines all filters together.
-- Function exposed for testing purposes. See options.lua -> fullTest()
function IEE.ignoreEnemyEmotesFilter(race, language, inPvP)
  return isOpposingFaction(race, language) and filterForOutdoorSettings(inPvP)
end

-- Function Wrapper to comply with Blizzard UI.
-- This should call the filters used by this addon.
local function FilterOpposingFactionSpeech(self, event, msg, sender, language, ...)
	-- Get the glocal id of the sender.
  local guid = select(9,...)
	if guid then local race = select(4, GetPlayerInfoByGUID(guid))
    return IEE.ignoreEnemyEmotesFilter(race, language, isInBgOrArena())
	end
  return true -- Fallback, don't filter.
end

-------------------------------------------------------------------------------------
-- | Event Hooks and Handlers |------------------------------------------------------
-------------------------------------------------------------------------------------

-- Called when the Addon is loaded. Add initialization code here.
local function OnAddonLoad()
  -- Initialize the DB.
  IgnoreEnemyEmotesDB = IgnoreEnemyEmotesDB or { }
  -- Initialize options in options.lua.
  IEE.initializeOptionsMenu()
  
  -- Add filters to all relevant chats.
  ChatFrame_AddMessageEventFilter( "CHAT_MSG_EMOTE",      FilterOpposingFactionSpeech)
  ChatFrame_AddMessageEventFilter( "CHAT_MSG_TEXT_EMOTE",	FilterOpposingFactionSpeech)
  ChatFrame_AddMessageEventFilter( "CHAT_MSG_YELL",       FilterOpposingFactionSpeech)
  ChatFrame_AddMessageEventFilter( "CHAT_MSG_SAY",       	FilterOpposingFactionSpeech)
end

-- Called when the Addon is unloaded. Add finalization code here.
local function OnAddonUnload()
  IEE.unloadOptions()
end


-- Register to the ADDON_LOADED and PLAYER_LEAVING_WORLD events.
-- PLAYER_LEAVING_WORLD is used over PLAYER_LOGOUT due reload not calling PLAYER_LOGOUT.
local eventFrame = CreateFrame("FRAME")

eventFrame:RegisterEvent("ADDON_LOADED")
eventFrame:RegisterEvent("PLAYER_LEAVING_WORLD")

-- Event Handler function to respond to ADDON_LOADED and PLAYER_LEAVING_WORLD.
function eventFrame.OnEvent(unusedArg, event, name)
  if name == "IgnoreEnemyEmotes" then
    if     event == "ADDON_LOADED"          then OnAddonLoad() end
  end
  if event == "PLAYER_LEAVING_WORLD"        then OnAddonUnload()      end
end

eventFrame:SetScript("OnEvent", eventFrame.OnEvent)

-- eof