-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, IEE = ...

-- Create the options namespace.
local options = { }

-- Expose the options to the IgnoreEnemyEmotes Addon Space.
IEE.options = options

-------------------------------------------------------------------------------------
-- | Variable Definitions |----------------------------------------------------------
-------------------------------------------------------------------------------------

-- Active options, these represent the live, non-cache states of the addon.
-- These should be read from the IgnoreEnemyEmotesDB on loading, and flushed to it on unloading.
-- These should be read from the cache on confirming changes.
options.savedVars = { }
options.savedVars["onlyInPvP"] = false

-- Cached options.
-- These should be flushed to the savedVars on confirming changes.
-- These should be cleared when leaving the options menu.
-- Could make these local if event handling of options.xml is moved to this file. For now, insufficient reason to do so.
-- Preferably define these variables explicitly for every savedVar, even if eventhandling may automatically generate them.
options.cache = { }
options.cache["onlyInPvP"] = nil

-- Default options. These cannot be changed after being set.
-- These must be explicitly defined for every savedVar.
local defaults = { }
defaults["onlyInPvP"] = false

-------------------------------------------------------------------------------------
-- | Event Hooks and Handlers |------------------------------------------------------
-------------------------------------------------------------------------------------

-- Initializes the options menu.
function IEE.initializeOptionsMenu()
  
  -- Prepare the options panel for adding to the Interface->Addons menu.
  optionsPanel          = IgnoreEnemyEmotesOptionsPanel
  optionsPanel.name     = "IgnoreEnemyEmotes"
  
  -- On okay, flush all changes in cache to savedVars.
  optionsPanel.okay     = function()
    for k,v in pairs(options.cache) do
      options.savedVars[k] = v
      options.cache  [k] = nil
    end
  end
  
  -- On default, reset all in live to the default.
  optionsPanel.default  = function()
    for k,v in pairs(options.cache) do
      options.savedVars[k] = defaults[k]
    end
	end
  
  -- On cancel, reset the cache.
  optionsPanel.cancel   = function()
    for k,_ in pairs(options.cache) do
      options.cache[k] = nil
    end
  end
  
  -- Add the options panel to the Interface->Addons menu.
	InterfaceOptions_AddCategory(optionsPanel)
  
  -- Read all savedVars from either IgnoreEnemyEmotesDB or their default value.
  for k, _ in pairs(defaults) do
    options.savedVars[k] = IgnoreEnemyEmotesDB[k] or defaults[k]
  end
end

-- Flushes all options.savedVars to IgnoreEnemyEmotesDB.
-- To be called on a reload or logout.
-- Exposed to Addon to let IgnoreEnemyEmotes.lua call it.
function IEE.unloadOptions()
  -- Write all savedVars to IgnoreEnemyEmotesDB
  for k, _ in pairs(options.savedVars) do
    IgnoreEnemyEmotesDB[k] = options.savedVars[k]
  end
  
end

-------------------------------------------------------------------------------------
-- | Testing Functionality |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Test function.
-- Could expand to only show failed cases.
options.fullTest = function()
  print("IEE Testing: Full Test")
   
  local pvpOptions  = { true, false }
  local optOptions  = { true, false }
  local raceOptions = { "Human", "Orc", "Pandaren" }
  local langOptions = { "Common", "Orcish" }
  
  local testID = 0
  local result = nil
  
  local onlyInPvPVar = options.savedVars["onlyInPvP"] -- store the original saved var as we'll adjust it during testing and have to set it back.
  
  -- perform test.
  for _, l in ipairs(langOptions) do
    for _, r in ipairs(raceOptions) do
      for _, o in ipairs(optOptions) do
        for _, p in ipairs(pvpOptions) do
          if  not ((l == "Common" and r == "Orc") 
              or   (l == "Orcish" and r == "Human")) then
            testID = testID + 1
            print("Test Case: " .. testID)
            print(p and "In PvP" or "Outdoors")
            print("Filter: " .. (o and "Only in PvP" or "Everywhere"))
            print("Race: " .. r)
            print("Language: " .. l)
            options.savedVars["onlyInPvP"] = o
            result = IEE.ignoreEnemyEmotesFilter(r, l, p)
            print("Message Filtered: " .. ((result) and "Yes" or "No"))
          end
        end
      end
    end
  end
  
  options.savedVars["onlyInPvP"] = onlyInPvPVar
end

-- eof